"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var makeError = require("make-error"), coreConstants = require("@remirror/core-constants"), _slicedToArray = require("@babel/runtime/helpers/slicedToArray"), _createForOfIteratorHelper = require("@babel/runtime/helpers/createForOfIteratorHelper"), _objectSpread = require("@babel/runtime/helpers/objectSpread2"), deepmerge = require("deepmerge"), fastDeepEqual = require("fast-deep-equal"), omit = require("object.omit"), object_pick = require("object.pick"), caseAnything = require("case-anything"), throttleDebounce = require("throttle-debounce");

function _interopDefault(e) {
  return e && e.__esModule ? e : {
    default: e
  };
}

var TypeName, _slicedToArray__default = _interopDefault(_slicedToArray), _createForOfIteratorHelper__default = _interopDefault(_createForOfIteratorHelper), _objectSpread__default = _interopDefault(_objectSpread), deepmerge__default = _interopDefault(deepmerge), fastDeepEqual__default = _interopDefault(fastDeepEqual), omit__default = _interopDefault(omit), object_pick__default = _interopDefault(object_pick);

function Cast(parameter) {
  return parameter;
}

function entries(value) {
  return Object.entries(value);
}

function keys(value) {
  return Object.keys(value);
}

function values(value) {
  return Object.values(value);
}

function includes(array, item, fromIndex) {
  return array.includes(item, fromIndex);
}

function object(value) {
  return Object.assign(Object.create(null), value);
}

function bool(value) {
  return !!value;
}

function toString(value) {
  return Object.prototype.toString.call(value);
}

function not(predicate) {
  return a => !predicate(a);
}

function getObjectType(value) {
  return toString(value).slice(8, -1);
}

function isOfType(type, predicate) {
  return value => typeof value === type && (!predicate || predicate(value));
}

function isObjectOfType(type) {
  return value => getObjectType(value) === type;
}

function isDirectInstanceOf(instance, Constructor) {
  return Object.getPrototypeOf(instance) === Constructor.prototype;
}

!function(TypeName) {
  TypeName.Object = "Object", TypeName.RegExp = "RegExp", TypeName.Date = "Date", 
  TypeName.Promise = "Promise", TypeName.Error = "Error", TypeName.Map = "Map", TypeName.Set = "Set";
}(TypeName || (TypeName = {}));

var isUndefined = isOfType("undefined"), isString = isOfType("string"), isNumber = isOfType("number", (value => !Number.isNaN(value))), isFunction = isOfType("function");

function isNull(value) {
  return null === value;
}

function isClass(value) {
  return isFunction(value) && value.toString().startsWith("class ");
}

function isBoolean(value) {
  return !0 === value || !1 === value;
}

var isSymbol = isOfType("symbol");

function isInteger(value) {
  return Number.isInteger(value);
}

function isSafeInteger(value) {
  return Number.isSafeInteger(value);
}

function isPlainObject(value) {
  if (getObjectType(value) !== TypeName.Object) return !1;
  var prototype = Object.getPrototypeOf(value);
  return null === prototype || prototype === Object.getPrototypeOf({});
}

function isPrimitive(value) {
  return null == value || /^[bns]/.test(typeof value);
}

function isNullOrUndefined(value) {
  return isNull(value) || isUndefined(value);
}

function isObject(value) {
  return !isNullOrUndefined(value) && (isFunction(value) || isOfType("object")(value));
}

function isInstanceOf(Constructor) {
  return value => isObject(value) && value instanceof Constructor;
}

function isNativePromise(value) {
  return isObjectOfType(TypeName.Promise)(value);
}

var hasPromiseAPI = value => !(isNull(value) || !isObject(value) || !isFunction(value.then) || !isFunction(value.catch));

function isPromise(value) {
  return isNativePromise(value) || hasPromiseAPI(value);
}

var isRegExp = isObjectOfType(TypeName.RegExp), isDate = isObjectOfType(TypeName.Date), isError = isObjectOfType(TypeName.Error);

function isMap(value) {
  return isObjectOfType(TypeName.Map)(value);
}

function isSet(value) {
  return isObjectOfType(TypeName.Set)(value);
}

function isEmptyObject(value) {
  return isObject(value) && !isMap(value) && !isSet(value) && 0 === Object.keys(value).length;
}

var isArray = Array.isArray;

function isEmptyArray(value) {
  return isArray(value) && 0 === value.length;
}

var isRemirrorType = value => isObject(value);

function isIdentifierOfType(value, type) {
  return isArray(type) ? includes(type, value[coreConstants.__INTERNAL_REMIRROR_IDENTIFIER_KEY__]) : type === value[coreConstants.__INTERNAL_REMIRROR_IDENTIFIER_KEY__];
}

function capitalize(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
}

function format(value) {
  return value = value.trim(), /^(?:webOS|i(?:OS|P))/.test(value) ? value : capitalize(value);
}

function callIfDefined(fn) {
  if (isFunction(fn)) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) args[_key - 1] = arguments[_key];
    fn(...args);
  }
}

function findMatches(text, regexp) {
  var match, runWhile = arguments.length > 2 && void 0 !== arguments[2] ? arguments[2] : match => bool(match), results = [], flags = regexp.flags;
  flags.includes("g") || (regexp = new RegExp(regexp.source, "g".concat(flags)));
  do {
    (match = regexp.exec(text)) && results.push(match);
  } while (runWhile(match));
  return regexp.lastIndex = 0, results;
}

function cleanupOS(os, pattern, label) {
  return pattern && label && (os = os.replace(new RegExp(pattern, "i"), label)), format(os.replace(/ ce$/i, " CE").replace(/\bhpw/i, "web").replace(/\bMacintosh\b/, "Mac OS").replace(/_powerpc\b/i, " OS").replace(/\b(os x) [^\d ]+/i, "$1").replace(/\bMac (OS X)\b/, "$1").replace(/\/(\d)/, " $1").replace(/_/g, ".").replace(/(?: bepc|[ .]*fc[\d .]+)$/i, "").replace(/\bx86\.64\b/gi, "x86_64").replace(/\b(Windows Phone) OS\b/, "$1").replace(/\b(Chrome OS \w+) [\d.]+\b/, "$1").split(" on ")[0]);
}

function isAndroidOS() {
  var ua = navigator.userAgent, match = new RegExp("\\bAndroid(?:/[\\d.]+|[ \\w.]*)", "i").exec(ua);
  return !!match && cleanupOS(match[0], "Android", "Android").includes("Android");
}

function randomFloat(min, max) {
  return max || (max = min, min = 0), Math.random() * (max - min + 1) + min;
}

function randomInt(min, max) {
  return Math.floor(randomFloat(min, max));
}

function startCase(string) {
  return string.replace(/_/g, " ").replace(/([a-z])([A-Z])/g, ((_, $1, $2) => "".concat($1, " ").concat($2))).replace(/(\s|^)(\w)/g, ((_, $1, $2) => "".concat($1).concat($2.toUpperCase())));
}

function n() {
  var time = Date.now(), last = n.last || time;
  return n.last = time > last ? time : last + 1;
}

function uniqueId() {
  var prefix = arguments.length > 0 && void 0 !== arguments[0] ? arguments[0] : "";
  return "".concat(prefix).concat(n().toString(36));
}

function take(array, number) {
  return number = Math.max(Math.min(0, number), number), array.slice(0, number);
}

function omitUndefined(object) {
  return omit__default.default(object, (value => !isUndefined(value)));
}

function clone(value) {
  if (!isPlainObject(value)) throw new Error("An invalid value was passed into this clone utility. Expected a plain object");
  return _objectSpread__default.default({}, value);
}

function shallowClone(value) {
  var clone = Object.create(Object.getPrototypeOf(value)), descriptors = Object.getOwnPropertyDescriptors(value);
  return Object.defineProperties(clone, descriptors), clone;
}

n.last = 0;

var isEqual = fastDeepEqual__default.default;

function uniqueArray(array) {
  var fromStart = arguments.length > 1 && void 0 !== arguments[1] && arguments[1], array_ = fromStart ? [ ...array ].reverse() : array, set = new Set(array_);
  return fromStart ? [ ...set ].reverse() : [ ...set ];
}

function flattenArray(array) {
  var _step, flattened = [], _iterator = _createForOfIteratorHelper__default.default(array);
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done; ) {
      var _item = _step.value, itemsToInsert = isArray(_item) ? flattenArray(_item) : [ _item ];
      flattened.push(...itemsToInsert);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return flattened;
}

function noop() {}

function deepMerge() {
  for (var _len2 = arguments.length, objects = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) objects[_key2] = arguments[_key2];
  return deepmerge__default.default.all(objects, {
    isMergeableObject: isPlainObject
  });
}

function clamp(_ref) {
  var min = _ref.min, max = _ref.max, value = _ref.value;
  return value < min ? min : value > max ? max : value;
}

function last(array) {
  return array[array.length - 1];
}

function sort(array, compareFn) {
  return [ ...array ].map(((value, index) => ({
    value: value,
    index: index
  }))).sort(((a, z) => compareFn(a.value, z.value) || a.index - z.index)).map((_ref2 => _ref2.value));
}

function get(path, obj, fallback) {
  if (!path || isEmptyArray(path)) return isUndefined(obj) ? fallback : obj;
  isString(path) && (path = path.split("."));
  for (var ii = 0, length_ = path.length; ii < length_ && obj; ++ii) {
    if (!isPlainObject(obj) && !isArray(obj)) return fallback;
    obj = obj[path[ii]];
  }
  return isUndefined(obj) ? fallback : obj;
}

function setPropInternal(path, obj, value, index) {
  if (path.length === index) return value;
  obj = obj || {};
  var key = path[index];
  return setClone(obj, key, setPropInternal(path, obj[key], value, ++index));
}

function setClone(obj, key, value) {
  var newObj = clone(obj);
  return newObj[key] = value, newObj;
}

function set(path, obj, value) {
  return isNumber(path) ? setClone(obj, path, value) : (isString(path) && (path = path.split(".")), 
  setPropInternal(path, obj, value, 0));
}

function unset(path, target) {
  var _step2, clonedObject = clone(target), value = clonedObject, _iterator2 = _createForOfIteratorHelper__default.default(path.entries());
  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done; ) {
      var _step2$value = _slicedToArray__default.default(_step2.value, 2), index = _step2$value[0], key = _step2$value[1], shouldDelete = index >= path.length - 1, _item2 = value[key];
      if (shouldDelete) {
        if (isArray(value)) {
          var indexKey = Number.parseInt(key.toString(), 10);
          isNumber(indexKey) && value.splice(indexKey, 1);
        } else Reflect.deleteProperty(value, key);
        return clonedObject;
      }
      if (isPrimitive(_item2)) return clonedObject;
      _item2 = isArray(_item2) ? [ ..._item2 ] : _objectSpread__default.default({}, _item2), 
      value[key] = _item2, value = _item2;
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }
  return clonedObject;
}

function makeFunctionForUniqueBy(value) {
  return item => get(value, item);
}

function uniqueBy(array, getValue) {
  var _step3, fromStart = arguments.length > 2 && void 0 !== arguments[2] && arguments[2], unique = [], found = new Set, getter = isFunction(getValue) ? getValue : makeFunctionForUniqueBy(getValue), list = fromStart ? [ ...array ].reverse() : array, _iterator3 = _createForOfIteratorHelper__default.default(list);
  try {
    for (_iterator3.s(); !(_step3 = _iterator3.n()).done; ) {
      var _item3 = _step3.value, _value = getter(_item3);
      found.has(_value) || (found.add(_value), unique.push(_item3));
    }
  } catch (err) {
    _iterator3.e(err);
  } finally {
    _iterator3.f();
  }
  return fromStart ? unique.reverse() : unique;
}

function range(start, end) {
  return isNumber(end) ? start <= end ? Array.from({
    length: end + 1 - start
  }, ((_, index) => index + start)) : Array.from({
    length: start + 1 - end
  }, ((_, index) => -1 * index + start)) : Array.from({
    length: Math.abs(start)
  }, ((_, index) => (start < 0 ? -1 : 1) * index));
}

function within(value) {
  for (var _len3 = arguments.length, rest = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) rest[_key3 - 1] = arguments[_key3];
  var numbers = rest.filter(isNumber);
  return value >= Math.min(...numbers) && value <= Math.max(...numbers);
}

function hasOwnProperty(object_, key) {
  return Object.prototype.hasOwnProperty.call(object_, key);
}

function getLazyArray(value) {
  return isFunction(value) ? value() : value;
}

var ERROR_INFORMATION_URL = "https://remirror.io/docs/errors", errorMessageMap = {};

function isErrorConstant(code) {
  return isString(code) && includes(values(coreConstants.ErrorConstant), code);
}

function createErrorMessage(code, extraMessage) {
  var message = errorMessageMap[code], prefix = message ? "".concat(message, "\n\n") : "", customMessage = extraMessage ? "".concat(extraMessage, "\n\n") : "";
  return "".concat(prefix).concat(customMessage, "For more information visit ").concat(ERROR_INFORMATION_URL, "#").concat(code.toLowerCase());
}

class RemirrorError extends makeError.BaseError {
  static create() {
    return new RemirrorError(arguments.length > 0 && void 0 !== arguments[0] ? arguments[0] : {});
  }
  constructor() {
    var errorCode, _ref = arguments.length > 0 && void 0 !== arguments[0] ? arguments[0] : {}, code = _ref.code, message = _ref.message, _ref$disableLogging = _ref.disableLogging, disableLogging = void 0 !== _ref$disableLogging && _ref$disableLogging;
    isErrorConstant(code) ? super(createErrorMessage(errorCode = code, message)) : super(createErrorMessage(errorCode = coreConstants.ErrorConstant.CUSTOM, message)), 
    this.errorCode = errorCode, this.url = "".concat(ERROR_INFORMATION_URL, "#").concat(errorCode.toLowerCase()), 
    disableLogging || console.error(this.message);
  }
}

function invariant(condition, options) {
  if (!condition) throw RemirrorError.create({
    code: coreConstants.ErrorConstant.PROD
  });
}

var freeze = function(target) {
  return target;
};

Object.defineProperty(exports, "omit", {
  enumerable: !0,
  get: function() {
    return omit__default.default;
  }
}), Object.defineProperty(exports, "pick", {
  enumerable: !0,
  get: function() {
    return object_pick__default.default;
  }
}), Object.defineProperty(exports, "camelCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.camelCase;
  }
}), Object.defineProperty(exports, "capitalCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.capitalCase;
  }
}), Object.defineProperty(exports, "constantCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.constantCase;
  }
}), Object.defineProperty(exports, "kebabCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.kebabCase;
  }
}), Object.defineProperty(exports, "pascalCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.pascalCase;
  }
}), Object.defineProperty(exports, "pathCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.pathCase;
  }
}), Object.defineProperty(exports, "snakeCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.snakeCase;
  }
}), Object.defineProperty(exports, "spaceCase", {
  enumerable: !0,
  get: function() {
    return caseAnything.spaceCase;
  }
}), Object.defineProperty(exports, "debounce", {
  enumerable: !0,
  get: function() {
    return throttleDebounce.debounce;
  }
}), Object.defineProperty(exports, "throttle", {
  enumerable: !0,
  get: function() {
    return throttleDebounce.throttle;
  }
}), exports.Cast = Cast, exports.RemirrorError = RemirrorError, exports.bool = bool, 
exports.callIfDefined = callIfDefined, exports.capitalize = capitalize, exports.clamp = clamp, 
exports.cleanupOS = cleanupOS, exports.clone = clone, exports.deepMerge = deepMerge, 
exports.entries = entries, exports.findMatches = findMatches, exports.flattenArray = flattenArray, 
exports.format = format, exports.freeze = freeze, exports.get = get, exports.getLazyArray = getLazyArray, 
exports.hasOwnProperty = hasOwnProperty, exports.includes = includes, exports.invariant = invariant, 
exports.isAndroidOS = isAndroidOS, exports.isArray = isArray, exports.isBoolean = isBoolean, 
exports.isClass = isClass, exports.isDate = isDate, exports.isDirectInstanceOf = isDirectInstanceOf, 
exports.isEmptyArray = isEmptyArray, exports.isEmptyObject = isEmptyObject, exports.isEqual = isEqual, 
exports.isError = isError, exports.isFunction = isFunction, exports.isIdentifierOfType = isIdentifierOfType, 
exports.isInstanceOf = isInstanceOf, exports.isInteger = isInteger, exports.isMap = isMap, 
exports.isNativePromise = isNativePromise, exports.isNull = isNull, exports.isNullOrUndefined = isNullOrUndefined, 
exports.isNumber = isNumber, exports.isObject = isObject, exports.isPlainObject = isPlainObject, 
exports.isPrimitive = isPrimitive, exports.isPromise = isPromise, exports.isRegExp = isRegExp, 
exports.isRemirrorType = isRemirrorType, exports.isSafeInteger = isSafeInteger, 
exports.isSet = isSet, exports.isString = isString, exports.isSymbol = isSymbol, 
exports.isUndefined = isUndefined, exports.keys = keys, exports.last = last, exports.noop = noop, 
exports.not = not, exports.object = object, exports.omitUndefined = omitUndefined, 
exports.randomFloat = randomFloat, exports.randomInt = randomInt, exports.range = range, 
exports.set = set, exports.shallowClone = shallowClone, exports.sort = sort, exports.startCase = startCase, 
exports.take = take, exports.toString = toString, exports.uniqueArray = uniqueArray, 
exports.uniqueBy = uniqueBy, exports.uniqueId = uniqueId, exports.unset = unset, 
exports.values = values, exports.within = within;
